c===============================================================================

      subroutine reinhard

c===============================================================================
c      calculates diffraction radius and surface thickness
c      ref. Z. Phys. A 323, 13 (1986)
c-------------------------------------------------------------------------------
      implicit real*8 (a-h,o-z)
      include 'dirhb.par'



      logical fc_rise,fc_zero,fc_max
      dimension ap(1:4),an(1:4),bp(1:4),bn(1:4)

      common /baspar/ hom,hb0,b0
      common /gaucor/ wdcor(MG)
      common /gaussh/ xh(0:NGH),wh(0:NGH),zb(0:NGH)
      common /gaussl/ xl(0:NGL),wl(0:NGL),sxl(0:NGL),rb(0:NGL)
      common /mathco/ zero,one,two,half,third,pi
      common /nucnuc/ amas,nama,npr(2),nucnam 
      common /physco/ hqc,alphi,r0
      common /masses/ amu(2),amsig,amome,amrho
      common /rhorho/ rs(MG,2),rv(MG,2)
      common /erwar / ea,rms,betg,gamg,diffr,surface
      common /centma/ cmas(3)

      ihl(ih,il) = 1+ih + il*(NGH+1)

      data mmaxr/256/,rstep/0.05/,maxi/4000/,qqstep/0.001/
      data ap /0.312, 1.312, -0.709, 0.085/
      data bp /0.16667,0.06658,0.02269,0.006485/
      data an /1.,-1.,0.,0./
      data bn /0.04833,0.05833,0.0,0.0/

      amu(1) = amu(1)*hqc
      amu(2) = amu(2)*hqc

      cmwid = 1.58/(6.0*(amas)**0.66666667)
      fc_rise = .false.
      fc_zero = .false.
      fc_max = .false.

      amun_fm = amu(1)/hqc  !nucleon mass in fm^{-1}
      amup_fm = amu(2)/hqc
      facn = one/(two*amun_fm*amas)
      facn = facn*hqc    ! correction in MeV

      facp = one/(two*amup_fm*amas)
      facp = facp*hqc    ! correction in MeV
 
      p2n = -1.0*cmas(1)/facn
      p2p = -1.0*cmas(2)/facp

      do i=1,maxi
         q=i*qqstep
c      radii only for spherical nuclei       
c----- radial integrals (2.15)
         fnq=zero
         fpq=zero
          do ih=1,NGH
             z=zb(ih)
             zz=z**2
         do il=1,NGL
            rrp=rb(il)**2
            rr = zz + rrp
            r = sqrt(rr)
c            wx=b0*wl(il)
            ii=ihl(ih,il)
            rintn=sin(q*r)*rv(ii,1)/r
            rintp=sin(q*r)*rv(ii,2)/r
            fnq=fnq+rintn
            fpq=fpq+rintp
         enddo
         enddo
         fnq=4.*pi*fnq/q
         fpq=4.*pi*fpq/q

c----- factors 2.16
         ffnq=zero
	 ffpq=zero
	 do j=1,4
	    ffnq=ffnq+an(j)/(1+bn(j)*q**2)
	    ffpq=ffpq+ap(j)/(1+bp(j)*q**2)
	 enddo   

c----- F_c
c        fcq=(fpq*ffpq+fnq*ffnq)*exp((0.51*q)**2/amas**(2./3.))
c        fcq=(fpq*ffpq+fnq*ffnq)*exp(q**2/(8.0*p2p)) ! true CM correction
         fcq=fpq*ffpq*exp(q**2/(8.0*p2p))+fnq*ffnq*exp(q**2/(8.0*p2n))
c        fcq=(fpq*ffpq+fnq*ffnq)*exp((q**2)*cmwid) ! true CM correction
c----- search fort F_c=0.0 and for max. F_c
         if (i .eq. 1) then
            fcq_old = fcq
            fcq_norm = fcq
         else
            if (fc_zero) then
               goto 15
            else
               if ( (fcq*fcq_old) .lt. zero) then
                  q_zero = q - qqstep/2.
                  fc_zero = .true.
               endif
            endif
15          continue
            if (fc_max) then
               goto 17
            else
               if (fc_rise) then
                  if (fcq .lt. fcq_old) then
                     qqmax = q - qqstep
                     fc_m = fcq_old
                     fc_max = .true.
                  endif
               endif
               if (fcq_old .lt. fcq) fc_rise = .true.
            endif
         endif
17       continue
         if (fc_zero .and. fc_max) goto 10
         fcq_old = fcq
      enddo
10    continue

c----- diffraction radius (2.17)
      diffr = 4.493/q_zero  
       
c----- form factor normalization F_c(0) = 1
      fc_m = fc_m/fcq_norm 

c----- surface thickness (2.18)
      x = qqmax*diffr
      t1 = 1/x*(-cos(x)+sin(x)/x)
      t2 = 3.d0*t1/(x*fc_m)
      surface = sqrt(two*log(t2)/qqmax**2)
      write(*,*)'diffr,surface', diffr,surface

      return
      end
                           
